@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2008-2024 Jonathan Schleifer <js@nil.im>
+ * Copyright (c) 2008-2025 Jonathan Schleifer <js@nil.im>
  *
  * All rights reserved.
  *
@@ -25,6 +25,8 @@
 #import "OFConcreteArray.h"
 #import "OFArray+Private.h"
 #import "OFData.h"
+#import "OFIndexSet.h"
+#import "OFIndexSet+Private.h"
 
 #import "OFEnumerationMutationException.h"
 #import "OFInvalidArgumentException.h"
@@ -45,7 +47,7 @@
 		_array = [[OFMutableData alloc] initWithItemSize: sizeof(id)
 							capacity: capacity];
 	} @catch (id e) {
-		[self release];
+		objc_release(self);
 		@throw e;
 	}
 
@@ -58,7 +60,7 @@
 		@throw [OFInvalidArgumentException exception];
 
 	[_array addItem: &object];
-	[object retain];
+	objc_retain(object);
 
 	_mutations++;
 }
@@ -68,12 +70,9 @@
 	if (object == nil)
 		@throw [OFInvalidArgumentException exception];
 
-	@try {
-		[_array insertItem: &object atIndex: idx];
-	} @catch (OFOutOfRangeException *e) {
-		@throw [OFOutOfRangeException exception];
-	}
-	[object retain];
+	[_array insertItem: &object atIndex: idx];
+
+	objc_retain(object);
 
 	_mutations++;
 }
@@ -83,14 +82,26 @@
 	id const *objects = array.objects;
 	size_t count = array.count;
 
-	@try {
-		[_array insertItems: objects atIndex: idx count: count];
-	} @catch (OFOutOfRangeException *e) {
-		@throw [OFOutOfRangeException exception];
-	}
+	[_array insertItems: objects atIndex: idx count: count];
 
 	for (size_t i = 0; i < count; i++)
-		[objects[i] retain];
+		objc_retain(objects[i]);
+
+	_mutations++;
+}
+
+- (void)insertObjects: (OFArray *)array atIndexes: (OFIndexSet *)indexes
+{
+	id const *objects = array.objects;
+	size_t count = array.count;
+
+	if (indexes.count != count)
+		@throw [OFOutOfRangeException exception];
+
+	[_array insertItems: objects atIndexes: indexes];
+
+	for (size_t i = 0; i < count; i++)
+		objc_retain(objects[i]);
 
 	_mutations++;
 }
@@ -108,8 +119,8 @@
 
 	for (size_t i = 0; i < count; i++) {
 		if ([objects[i] isEqual: oldObject]) {
-			[newObject retain];
-			[objects[i] release];
+			objc_retain(newObject);
+			objc_release(objects[i]);
 			objects[i] = newObject;
 		}
 	}
@@ -129,8 +140,8 @@
 		@throw [OFOutOfRangeException exception];
 
 	oldObject = objects[idx];
-	objects[idx] = [object retain];
-	[oldObject release];
+	objects[idx] = objc_retain(object);
+	objc_release(oldObject);
 }
 
 - (void)replaceObjectIdenticalTo: (id)oldObject withObject: (id)newObject
@@ -146,8 +157,8 @@
 
 	for (size_t i = 0; i < count; i++) {
 		if (objects[i] == oldObject) {
-			[newObject retain];
-			[objects[i] release];
+			objc_retain(newObject);
+			objc_release(objects[i]);
 			objects[i] = newObject;
 
 			return;
@@ -155,6 +166,33 @@
 	}
 }
 
+- (void)replaceObjectsAtIndexes: (OFIndexSet *)indexes
+		    withObjects: (OFArray *)objects
+{
+	void *pool = objc_autoreleasePoolPush();
+	const OFRange *ranges = indexes.of_ranges.items;
+	size_t rangesCount = indexes.of_ranges.count;
+	id const *objectsObjects = objects.objects;
+	size_t objectsCount = objects.count;
+	size_t objectsIndex = 0;
+	id *items = _array.mutableItems;
+	size_t count = _array.count;
+
+	if (objectsCount != indexes.count)
+		@throw [OFOutOfRangeException exception];
+
+	for (size_t i = 0; i < rangesCount; i++) {
+		if (OFEndOfRange(ranges[i]) > count)
+			@throw [OFOutOfRangeException exception];
+
+		for (size_t j = ranges[i].location; j < OFEndOfRange(ranges[i]);
+		    j++)
+			items[j] = objectsObjects[objectsIndex++];
+	}
+
+	objc_autoreleasePoolPop(pool);
+}
+
 - (void)removeObject: (id)object
 {
 	id const *objects;
@@ -173,7 +211,7 @@
 			[_array removeItemAtIndex: i];
 			_mutations++;
 
-			[tmp release];
+			objc_release(tmp);
 
 			objects = _array.items;
 			i--;
@@ -199,7 +237,7 @@
 			[_array removeItemAtIndex: i];
 			_mutations++;
 
-			[object release];
+			objc_release(object);
 
 			objects = _array.items;
 			i--;
@@ -214,7 +252,7 @@
 #ifndef __clang_analyzer__
 	id object = [self objectAtIndex: idx];
 	[_array removeItemAtIndex: idx];
-	[object release];
+	objc_release(object);
 
 	_mutations++;
 #endif
@@ -226,7 +264,7 @@
 	size_t count = _array.count;
 
 	for (size_t i = 0; i < count; i++)
-		[objects[i] release];
+		objc_release(objects[i]);
 
 	[_array removeAllItems];
 }
@@ -237,19 +275,50 @@
 	size_t count = _array.count;
 	id *copy;
 
-	if (range.length > SIZE_MAX - range.location ||
-	    range.location >= count || range.length > count - range.location)
+	if (OFEndOfRange(range) > count)
 		@throw [OFOutOfRangeException exception];
 
 	copy = OFAllocMemory(range.length, sizeof(*copy));
-	memcpy(copy, objects + range.location, range.length * sizeof(id));
+	memcpy(copy, objects + range.location, range.length * sizeof(*copy));
 
 	@try {
 		[_array removeItemsInRange: range];
 		_mutations++;
 
 		for (size_t i = 0; i < range.length; i++)
-			[copy[i] release];
+			objc_release(copy[i]);
+	} @finally {
+		OFFreeMemory(copy);
+	}
+}
+
+- (void)removeObjectsAtIndexes: (OFIndexSet *)indexes
+{
+	size_t indexesCount = indexes.count;
+	const OFRange *ranges = indexes.of_ranges.items;
+	size_t rangesCount = indexes.of_ranges.count;
+	id const *objects = _array.items;
+	size_t count = _array.count;
+	size_t copyIndex = 0;
+	id *copy;
+
+	copy = OFAllocMemory(indexesCount, sizeof(*copy));
+
+	for (size_t i = 0; i < rangesCount; i++) {
+		if (OFEndOfRange(ranges[i]) > count)
+			@throw [OFOutOfRangeException exception];
+
+		memcpy(copy + copyIndex, objects + ranges[i].location,
+		    ranges[i].length * sizeof(*copy));
+		copyIndex += ranges[i].length;
+	}
+
+	@try {
+		[_array removeItemsAtIndexes: indexes];
+		_mutations++;
+
+		for (size_t i = 0; i < indexesCount; i++)
+			objc_release(copy[i]);
 	} @finally {
 		OFFreeMemory(copy);
 	}
@@ -266,7 +335,7 @@
 
 	object = [self objectAtIndex: count - 1];
 	[_array removeLastItem];
-	[object release];
+	objc_release(object);
 
 	_mutations++;
 #endif
@@ -332,9 +401,9 @@
 
 - (OFEnumerator *)objectEnumerator
 {
-	return [[[OFArrayEnumerator alloc]
-	    initWithArray: self
-	     mutationsPtr: &_mutations] autorelease];
+	return objc_autoreleaseReturnValue(
+	    [[OFArrayEnumerator alloc] initWithArray: self
+					mutationsPtr: &_mutations]);
 }
 
 #ifdef OF_HAVE_BLOCKS
@@ -373,8 +442,8 @@
 			@throw [OFInvalidArgumentException exception];
 
 		if (new != objects[i]) {
-			[objects[i] release];
-			objects[i] = [new retain];
+			objc_release(objects[i]);
+			objects[i] = objc_retain(new);
 		}
 	}
 }
