@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2008-2024 Jonathan Schleifer <js@nil.im>
+ * Copyright (c) 2008-2025 Jonathan Schleifer <js@nil.im>
  *
  * All rights reserved.
  *
@@ -31,6 +31,7 @@
 #import "OFStreamSocket+Private.h"
 #import "OFRunLoop.h"
 #import "OFRunLoop+Private.h"
+#import "OFSocket.h"
 #import "OFSocket+Private.h"
 
 #import "OFAcceptSocketFailedException.h"
@@ -54,19 +55,9 @@
 @dynamic delegate;
 @synthesize listening = _listening;
 
-+ (void)initialize
-{
-	if (self != [OFStreamSocket class])
-		return;
-
-	if (!OFSocketInit())
-		@throw [OFInitializationFailedException
-		    exceptionWithClass: self];
-}
-
 + (instancetype)socket
 {
-	return [[[self alloc] init] autorelease];
+	return objc_autoreleaseReturnValue([[self alloc] init]);
 }
 
 - (instancetype)init
@@ -79,12 +70,16 @@
 			abort();
 		}
 
+		if (!_OFSocketInit())
+			@throw [OFInitializationFailedException
+			    exceptionWithClass: self.class];
+
 		_socket = OFInvalidSocketHandle;
 #ifdef OF_AMIGAOS
 		_socketID = -1;
 #endif
 	} @catch (id e) {
-		[self release];
+		objc_release(self);
 		@throw e;
 	}
 
@@ -119,7 +114,7 @@
 		@throw [OFReadFailedException
 		    exceptionWithObject: self
 			requestedLength: length
-				  errNo: OFSocketErrNo()];
+				  errNo: _OFSocketErrNo()];
 #else
 	if (length > INT_MAX)
 		@throw [OFOutOfRangeException exception];
@@ -128,7 +123,7 @@
 		@throw [OFReadFailedException
 		    exceptionWithObject: self
 			requestedLength: length
-				  errNo: OFSocketErrNo()];
+				  errNo: _OFSocketErrNo()];
 #endif
 
 	if (ret == 0)
@@ -153,7 +148,7 @@
 		    exceptionWithObject: self
 			requestedLength: length
 			   bytesWritten: 0
-				  errNo: OFSocketErrNo()];
+				  errNo: _OFSocketErrNo()];
 #else
 	int bytesWritten;
 
@@ -165,7 +160,7 @@
 		    exceptionWithObject: self
 			requestedLength: length
 			   bytesWritten: 0
-				  errNo: OFSocketErrNo()];
+				  errNo: _OFSocketErrNo()];
 #endif
 
 	return (size_t)bytesWritten;
@@ -183,7 +178,7 @@
 	if (ioctlsocket(_socket, FIONBIO, &v) == SOCKET_ERROR)
 		@throw [OFSetOptionFailedException
 		    exceptionWithObject: self
-				  errNo: OFSocketErrNo()];
+				  errNo: _OFSocketErrNo()];
 
 	_canBlock = canBlock;
 }
@@ -223,11 +218,11 @@
 - (int)of_socketError
 {
 	int errNo;
-	socklen_t len = sizeof(errNo);
+	socklen_t len = (socklen_t)sizeof(errNo);
 
 	if (getsockopt(_socket, SOL_SOCKET, SO_ERROR, (char *)&errNo,
 	    &len) != 0)
-		return OFSocketErrNo();
+		return _OFSocketErrNo();
 
 	return errNo;
 }
@@ -247,7 +242,7 @@
 		@throw [OFListenOnSocketFailedException
 		    exceptionWithSocket: self
 				backlog: backlog
-				  errNo: OFSocketErrNo()];
+				  errNo: _OFSocketErrNo()];
 
 	_listening = true;
 }
@@ -264,7 +259,7 @@
 	if (_socket == OFInvalidSocketHandle)
 		@throw [OFNotOpenException exceptionWithObject: self];
 
-	client = [[[[self class] alloc] init] autorelease];
+	client = objc_autorelease([[self.class alloc] init]);
 	client->_remoteAddress.length =
 	    (socklen_t)sizeof(client->_remoteAddress.sockaddr);
 
@@ -275,7 +270,7 @@
 	    OFInvalidSocketHandle)
 		@throw [OFAcceptSocketFailedException
 		    exceptionWithSocket: self
-				  errNo: OFSocketErrNo()];
+				  errNo: _OFSocketErrNo()];
 #elif defined(HAVE_ACCEPT4) && defined(SOCK_CLOEXEC)
 	if ((client->_socket = accept4(_socket,
 	    (struct sockaddr * )&client->_remoteAddress.sockaddr,
@@ -283,14 +278,14 @@
 	    OFInvalidSocketHandle)
 		@throw [OFAcceptSocketFailedException
 		    exceptionWithSocket: self
-				  errNo: OFSocketErrNo()];
+				  errNo: _OFSocketErrNo()];
 #else
 	if ((client->_socket = accept(_socket,
 	    (struct sockaddr *)&client->_remoteAddress.sockaddr,
 	    &client->_remoteAddress.length)) == OFInvalidSocketHandle)
 		@throw [OFAcceptSocketFailedException
 		    exceptionWithSocket: self
-				  errNo: OFSocketErrNo()];
+				  errNo: _OFSocketErrNo()];
 
 # if defined(HAVE_FCNTL) && defined(FD_CLOEXEC)
 	if ((flags = fcntl(client->_socket, F_GETFD, 0)) != -1)
@@ -326,6 +321,8 @@
 		break;
 	}
 
+	client.canBlock = self.canBlock;
+
 	return client;
 }
 
@@ -338,22 +335,46 @@
 {
 	[OFRunLoop of_addAsyncAcceptForSocket: self
 					 mode: runLoopMode
-					block: NULL
+				      handler: NULL
 				     delegate: _delegate];
 }
 
 #ifdef OF_HAVE_BLOCKS
 - (void)asyncAcceptWithBlock: (OFStreamSocketAsyncAcceptBlock)block
 {
-	[self asyncAcceptWithRunLoopMode: OFDefaultRunLoopMode block: block];
+	OFStreamSocketAcceptedHandler handler = ^ (OFStreamSocket *socket,
+	    OFStreamSocket *acceptedSocket, id exception) {
+		return block(acceptedSocket, exception);
+	};
+
+	[self asyncAcceptWithRunLoopMode: OFDefaultRunLoopMode
+				 handler: handler];
+}
+
+- (void)asyncAcceptWithHandler: (OFStreamSocketAcceptedHandler)handler
+{
+	[self asyncAcceptWithRunLoopMode: OFDefaultRunLoopMode
+				 handler: handler];
 }
 
 - (void)asyncAcceptWithRunLoopMode: (OFRunLoopMode)runLoopMode
 			     block: (OFStreamSocketAsyncAcceptBlock)block
+{
+	OFStreamSocketAcceptedHandler handler = ^ (OFStreamSocket *socket,
+	    OFStreamSocket *acceptedSocket, id exception) {
+		return block(acceptedSocket, exception);
+	};
+
+	[self asyncAcceptWithRunLoopMode: runLoopMode
+				 handler: handler];
+}
+
+- (void)asyncAcceptWithRunLoopMode: (OFRunLoopMode)runLoopMode
+			   handler: (OFStreamSocketAcceptedHandler)handler
 {
 	[OFRunLoop of_addAsyncAcceptForSocket: self
 					 mode: runLoopMode
-					block: block
+				      handler: handler
 				     delegate: nil];
 }
 #endif
