@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2008-2024 Jonathan Schleifer <js@nil.im>
+ * Copyright (c) 2008-2025 Jonathan Schleifer <js@nil.im>
  *
  * All rights reserved.
  *
@@ -40,7 +40,7 @@ static int dateTag;
 
 	value |= UINT64_C(4) << 60;
 
-	return OFFromBigEndianDouble(OFRawUInt64ToDouble(OFToBigEndian64(
+	return OFFromBigEndianDouble(OFBitConvertUInt64ToDouble(OFToBigEndian64(
 	    value)));
 }
 
