@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2008-2024 Jonathan Schleifer <js@nil.im>
+ * Copyright (c) 2008-2025 Jonathan Schleifer <js@nil.im>
  *
  * All rights reserved.
  *
@@ -30,10 +30,10 @@ OF_SUBCLASSING_RESTRICTED
 	bool _positive;
 }
 
-- (instancetype)initWithName: (OFString *)name
-		      analog: (bool)analog OF_UNAVAILABLE;
-- (instancetype)initWithAxis: (OHGameControllerAxis *)axis
-		    positive: (bool)positive OF_DESIGNATED_INITIALIZER;
+- (instancetype)oh_initWithName: (OFString *)name
+			 analog: (bool)analog OF_UNAVAILABLE;
+- (instancetype)oh_initWithAxis: (OHGameControllerAxis *)axis
+		       positive: (bool)positive OF_METHOD_FAMILY(init);
 @end
 
 OF_ASSUME_NONNULL_END
