@@ -33,6 +33,21 @@
 static OFString *
 buttonToName(uint16_t button, uint16_t vendorID, uint16_t productID)
 {
+	if (vendorID == OHVendorIDMicrosoft &&
+	    productID == OHProductIDXbox360WirelessReceiver) {
+		/*
+		 * D-Pad is reported as both HAT0X/HAT0Y and Trigger Happy 1-4.
+		 * Filter out Trigger Happy.
+		 */
+		switch (button) {
+		case BTN_TRIGGER_HAPPY1:
+		case BTN_TRIGGER_HAPPY2:
+		case BTN_TRIGGER_HAPPY3:
+		case BTN_TRIGGER_HAPPY4:
+			return nil;
+		}
+	}
+
 	switch (button) {
 	case BTN_A:
 		return @"A";
@@ -266,14 +281,14 @@ axisToName(uint16_t axis)
 		}
 		[axes makeImmutable];
 
-		_buttons = [buttons retain];
-		_axes = [axes retain];
+		_buttons = [buttons copy];
+		_axes = [axes copy];
 		_vendorID = vendorID;
 		_productID = productID;
 
 		objc_autoreleasePoolPop(pool);
 	} @catch (id e) {
-		[self release];
+		objc_release(self);
 		@throw e;
 	}
 
@@ -282,8 +297,8 @@ axisToName(uint16_t axis)
 
 - (void)dealloc
 {
-	[_buttons release];
-	[_axes release];
+	objc_release(_buttons);
+	objc_release(_axes);
 
 	[super dealloc];
 }
